'use strict'
# 그리기 영역 정보 
margin = new Margin(50)
[width, height] = [900, 700]

#------------------------------------------------------------
#
# ## appendPartition
#
# 파티션을 svg에 추가하는 함수 
#
# * @param  svg          d3.selection 메인이 될 svg요소 
# * @param  data         배열   데이터의 배열
# * @param  attrOrder    배열   요소의 순서 
# * @return d3.selection 파티션의 svg요소 
#
appendPartition = (svg, data, attrOrder)->
  # 파티션을 그리는 요소를 작성 
  main = svg
    .append("g").attr(
      width: width, height: height
      transform: "translate(" + margin.left + "," + margin.top + ")")

  # 색지정 
  color = d3.scale.category10()

  # 데이터를 중첩 구조로 변환. 그때 지불액의 집계도 한다(rollup)
  nest = d3.nest()
  for attr in attrOrder
    do (attr)->
      nest.key((d)-> d[attr])
  data = nest.rollup((values)-> d3.sum(values, (d)-> d.payment)).entries(users)

  # x/y 위치의 스케일을 작성 
  x = d3.scale.linear().range([0, width])
  y = d3.scale.linear().range([0, height])

  # 파티션의 layout을 작성 
  # children요소와 value요소 지정 
  partition = d3.layout.partition()
    .children((d)-> d.values)
    .value((d)-> 0|d.values) # 整数化

  # 중첩의 톱을 자식 요소와 같은 형식으로 변환 
  root = {key: "전체", values: data}

  # 노드의 위치 정보를 계산
  nodes = partition.nodes(root)

  # 노드 그리기 
  g = main.selectAll('g').data(nodes).enter().append('g')
    .classed((d)-> "depth#{d.depth}")
    .attr('transform', (d)-> "translate(#{x(d.y)},#{y(d.x)})")
  g.append('rect')
    .attr(
      width: (d)-> d.dy * width
      height: (d)-> d.dx * height
      fill: (d, idx)-> color(d.depth)
      stroke: 'grey'
      'data-val': (d)-> d.value
    )
    .style(
      opacity: (d)-> if d.parent then d.value / d.parent.value else 0.8
      cursor: 'pointer'
    )

  # 자릿수 구분 형식 작성 
  currency = d3.format(",")
  # 속성명과 총액 그리기 
  g.append('text').text((d)-> if d.key then "#{d.key} (#{currency(d.value)}엔)" else "" )
    .attr('dy', 12)
    .style('opacity', (d)-> if d.dx * height > 12 then 1 else 0)

  main

#------------------------------------------------------------
#
# ## appendController
#
# 컨트롤러를 추가한다
#
# * @param  svg          d3.selection 메인이 될 svg요소 
# * @param  attrOrder    배열           요소의 순서 
# * @return d3.selection 컨트롤러의 svg요소 
#
appendController = (svg, attrOrder)->
  # 색지정 
  color = d3.scale.category10()

  # 「전체」를 나타내는 요소를 추가 
  attrOrderWithTotal = ["전체"].concat(attrOrder)
  # 폭을 계산 
  w = width / attrOrderWithTotal.length
  # 삼각형의 path를 생성 
  triangle = d3.svg.symbol().type('triangle-up')

  # 컨트롤러의 svg요소를 추가 
  ctrl = svg
    .append("g").attr(
      width: width, height: margin.top
      transform: "translate(#{margin.left},0)")
  ctrlCell = ctrl.selectAll('g').data(attrOrderWithTotal).enter().append('g')
    .attr('transform', (d, idx)-> "translate(#{idx * w},0)")
  ctrlCell.append('rect')
    .attr(
      width: w
      height: margin.top - 10
      fill: (d, idx)-> color(idx)
      stroke: 'none'
    )
    .style('opacity', 0.5)
  # 컨트롤러의 각 요소(셀)에 좌삼각과 우삼각을 추가 
  ctrlCell.append('path').classed('toLeft', true)
    .attr(
      d: triangle
      transform: "translate(20, 20) rotate(-90)")
    .style(
      cursor: 'pointer'
      opacity: (d, idx)-> if idx <= 1 then 0 else 1)
  ctrlCell.append('path').classed('toRight', true)
    .attr(
      d: triangle
      transform: "translate(#{w - 20}, 20) rotate(90)")
    .style(
      cursor: 'pointer'
      opacity: (d, idx)-> if idx is 0 or idx is attrOrder.length then 0 else 1)

  ctrl

#------------------------------------------------------------
#
# ## 파티션 시각화 작성 
#
# * @param  target     문자열 svg요소를 추가할 DOM요소 
# * @param  data       배열   데이터의 배열 
# * @param  attrOrder  배열   요소의 순서 
# * @return null
#
createExploratoryPartition = (target, data = {}, attrOrder)->
  # svg 요소를 작성 
  svg = d3.select(target).append("svg")
    .attr(
      width: width + margin.width, height: height + margin.height)

  # 파티션을 작성한다 
  main = appendPartition(svg, data, attrOrder)
  # 컨트롤러를 작성한다 
  ctrl = appendController(svg, attrOrder)

  # 컨트롤러로 파티션의 순서를 교체할 수 있게 한다(왼쪽 요소와 교체)
  ctrl.selectAll('path.toLeft').on 'click', (d, idx)->
    attrOrder[idx - 2..idx - 1] = [attrOrder[idx - 1], attrOrder[idx - 2]]
    main.remove()
    main = appendPartition(svg, data, attrOrder)

  # 컨트롤러로 파티션의 순서를 표체할 수 있게 한다(오른쪽 요소와 교체)
  ctrl.selectAll('path.toRight').on 'click', (d, idx)->
    attrOrder[idx - 1..idx] = [attrOrder[idx], attrOrder[idx - 1]]
    main.remove()
    main = appendPartition(svg, data, attrOrder)

#------------------------------------------------------------
#
# # user의 데이터를 작성하고 지정한 순서로 탐색적 트리맵을 작성한다.
#
userNum = 3000
users = [0..userNum - 1].map (d)-> new User()
createExploratoryPartition('body', users, ['campaign', 'gender', 'age', 'job'])
